//
//  MBAnnotationView.h
//  MBMapKit
//
//  Copyright 2011 Mapbar Inc. All rights reserved.
//

#import <UIKit/UIKit.h>

/** 传递这个通知以再次请求弹出气泡的信息。 */
UIKIT_EXTERN NSString *MBAnnotationCalloutInfoDidChangeNotification;

enum
{
    MBAnnotationViewDragStateNone = 0,      /**< 视图不参与拖动操作。当拖动结束或取消后标注视图负责自己返回到这种状态 */
    MBAnnotationViewDragStateStarting,      /**< 一个动作发生表示视图开始拖动。地图视图根据适当的用户操作自动改变标注视图到这种状态 */
    MBAnnotationViewDragStateDragging,      /**< 表示视图在拖动操作中并正在跟踪进展 */
    MBAnnotationViewDragStateCanceling,     /**< 一个动作发生表示视图将要取消拖动操作，你可以把标注视图改为这种状态来取消操作 */
    MBAnnotationViewDragStateEnding         /**< 一个动作发生表示用户放下了视图，地图视图根据适当的用户操作自动改变标注视图到这种状态 */
};
/** @brief  表示标注视图当前的拖动状态。 */
typedef NSUInteger MBAnnotationViewDragState;


@protocol MBAnnotation;

/** @interface MBAnnotationView
 *
 *  @brief
 *      MBAnnotationView 类负责使标注在地图视图中可视。标注视图和标注对象是不精确配对的。当标注的坐标点在可视范围时，地图
 *      视图请求自己的委托提供对应的标注视图。稍后标注视图可能被回收，被放到一个由地图视图保持的复用队列中。
 *      \n设置标注视图的 image 属性是最有效的提供标注内容的方法。标注视图根据你指定的图片自动设置自己的大小，其内容绘制为图
 *      片。因为这是一个视图，所以你还可以重写 drawRect: 方法来手动绘制视图的内容。如果你选择直接重写 drawRect: 方法并且
 *      你没有给 image 属性指定自定义的图片，请注意标注视图的框架的宽度和高度默认设置为0。在你自定义的内容被绘制前，你必须修
 *      改视图框架属性使宽度和高度为非零值。一般来说，如果你的内容只是静态的图片，如果有需要改变 image 属性比你自己绘制图片
 *      效率更高。
 *      \n标注视图一直固定在地图视图中关联的标注指定点。虽然标注视图随地图滚动，标注视图位于一个独立的显示层，当地图的可视范
 *      围变化时标注视图并不会缩放。
 *      \n标注视图支持选择状态，这决定视图是未选中、选中还是选中同时显示弹出气泡。用户通过和标注视图的交互来切换选择状态。在
 *      未选中状态，标注视图不高亮显示。在选中状态，标注视图高亮但不显示弹出气泡。最终，标注可以同时高亮并显示弹出气泡。弹出视
 *      图显示标题字符串，控制显示更多的信息。标题的信息由标注对象提供，标注视图负责提供自定义的控件。
 *  @section 复用标注视图
 *  @details
 *      标注视图被设计成在用户改变地图可视范围能够被复用。地图视图的复用通过在滚动地图时避免创建新的视图显著地提高了性能。由于
 *      这个原因，标注视图不会和它关联的标注紧密连接。使用地图视图的属性（或者设置方法）来配置新标注对象的视图。
 *      \n任何你初始化标注视图的时候，你总需要给视图指定一个复用标识符。一旦标注视图不再被需要，地图视图可能把它们放到一个复
 *      用队列。一旦新的标注被添加到地图视图，然后委托对象可以调用 MBMapView的dequeueReusableAnnotationViewWithIdentifier 
 *      方法让一个已有的视图出列（而不是创建一个新的）并重新配置。
 *  @section 子类化注意
 *  @details
 *      你可以直接调用现有的 MBAnnotationView 类，或者如果有需要子类化 MBAnnotationView 来提供自定义行为。 MBAnnotationView 
 *      类的 image 属性允许你可以不子类化而直接设置标注视图的显示。为了方便起见你也可以创建自定义的子类，把标注视图设为一个已知
 *      的状态。
 *      \n子类化 MBAnnotationView 没有特殊的要求。不过下面的列表列出了一些你可能希望子类化的原因，以及一些在实现期望行为时
 *      需要重写的方法。
 *      \line 为了把标注视图置为一致的状态，你需要提供一个自定义的初始化方法。你自定义的初始化方法然后调用 initWithAnnotation:reuseIdentifier: 
 *      来初始化超类。
 *      \line 为了提供自定义的弹出气泡，重写 leftCalloutAccessoryView 方法并把它返回到视图。如果希望支持拖动标注视图，你的子
 *      类需要负责在拖动操作中的关键过渡时刻改变 dragState 属性到恰当值。欲了解更多信息，请参见 dragState 属性的描述。
 */
@interface MBAnnotationView : UIView
{
@private
    id <MBAnnotation> _annotation;
    
    NSString *_reuseIdentifier;
    UIImage *_image;
    CGPoint _centerOffset;
    CGPoint _calloutOffset;
    BOOL _enabled;
    BOOL _highlighted;
    BOOL _selected;
    BOOL _canShowCallout;   
    BOOL _draggable;
    MBAnnotationViewDragState _dragState;
}

/** 
 *
 *  @brief  初始化并返回一个新的标注视图。
 *  @param  annotation  和新视图关联的标注对象。
 *  @param  reuseIdentifier 如果你打算给相似类型的标注复用这个标注视图，指定一个字符串来标识它。即使你不打算复用视图可以
 *                          指定为nil，但一般还是推荐复用标注视图。
 *  @return 初始化的标注视图，或者当初始化对象出问题时则为nil.
 *  @note   
 *      复用标识符给你提供了一种通过回收滚入滚出地图的标注视图的方式来提高性能。当视图不再被需要时，它们被地图视图移动到一个复
 *      用队列。
*       \n当一个新的标注变为可视时，你的应用可以传一个合适的复用标识符给 MBMmapView 类的 dequeueReusableAnnotationViewWithIdentifier 
 *      方法来请求一个视图。
 */
- (id)initWithAnnotation:(id <MBAnnotation>)annotation reuseIdentifier:(NSString *)reuseIdentifier;

/**
 *
 *  @brief  标识这个标注视图可复用的字符串。
 *  @note   
 *      当你创建视图的时指定复用标识符。你可以调用这个类型来取得先前创建的但由于标注不在屏幕内而未使用的标注视图。
 *      \n如果你定义显然不同类型的标注（它们的标注视图也显然不同），你可以为每一种标注指定不同的复用标记来区分标注类型。
 */
@property (nonatomic, readonly) NSString *reuseIdentifier;

/**
 *
 *  @brief  当视图被移动到复用队列时调用。
 *  @note
 *      本方法的默认实现不执行任何操作。你可以在你自定义的标注视图中重写本方法，把视图在被返回给地图视图委托前设为一个已知状态。
 */
- (void)prepareForReuse;

/**
 *
 *  @brief   当前和视图关联的标注对象。
 *  @note 
 *      你不应该直接改变这个属性的数值。当标注视图在地图中可视的时候这个属性为非nil值。当视图排到复用队列准备复用时，属性值为nil.
 */
@property (nonatomic, retain) id <MBAnnotation> annotation;

/**
 *
 *  @brief  标注视图显示的图片。
 *  @note  
 *      分配一个新的图像给这个属性同时会改变视图的框架，这样视图可以和新图像的宽度和高度匹配。视图框架的位置不会改变。
 */
@property (nonatomic, retain) UIImage *image;

/**
 *
 *  @brief  显示视图时的偏移量（像素单位）。
 *  @note  
 *      默认标注视图的中心点放置在关联标注的坐标点。如果有需要你可以使用本属性改变标注视图的位置。这里的x和y偏移量以像素为单
 *      位。正的偏移量将标注视图右下移动，负的偏移量将标注视图左上移动。
 */
@property (nonatomic) CGPoint centerOffset;

/**
 *
 *  @brief  放置弹出气泡的偏移量（像素单位）。
 *  @note 
 *      这个属性决定移动弹出气泡的额外距离。当属性设置为(0,0)时，弹出气泡的固定点为标注视图框架的顶部中点。指定正的偏移量弹
 *      出气泡将右下移动，指定负的偏移量弹出气泡将左上移动。
 */
@property (nonatomic) CGPoint calloutOffset;

/**
 *
 *  @brief  指示标注是否启用的布尔值。
 *  @note 
 *      默认本属性的值为YES。如果这个属性的值为NO,标注视图将忽略触摸事件并且不能被选择。子类也可以根据本属性的值有差别地显示
 *      标注内容。
 */
@property (nonatomic, getter=isEnabled) BOOL enabled;

/**
 *
 *  @brief  指示标注视图是否高亮的布尔值。
 *  @note 
 *      你不应该直接设置这个属性的值。地图视图通过触摸事件进出标注视图的边界来设置它。
 */
@property (nonatomic, getter=isHighlighted) BOOL highlighted;

/**
 *
 *  @brief  指示标注视当前是否被选中的布尔值。
 *  @note   
 *      你不应该直接设置本属性的值。如果这个属性为YES，标注视图正显示着弹出气泡。
 */
@property (nonatomic, getter=isSelected) BOOL selected;

/**
 *
 *  @brief  设置标注视图的选择状态。
 *  @param  selected    如果希望视图被选中设为YES。
 *  @param  animated    选择状态的改变是动画的设为YES。
 *  @note   
 *      你不应该直接调用本方法。MBMapView 对象调用本方法来对应用户和标注的交互。
 */
- (void)setSelected:(BOOL)selected animated:(BOOL)animated;

/**
 *
 *  @brief  指示标注视图能否弹出气泡显示额外的信息。
 *  @see leftCalloutAccessoryView
 *  @see rightCalloutAccessoryView
 *  @note 
 *      如果YES,当用户点击选中标注视图时将显示一个标准的弹出气泡。弹出气泡使用关联标注对象的标题和子标题文本（如果标注的 title 
 *      方法返回空字符串，标注视图被视为它的 enabled 属性设置为NO）。弹出气泡同时还会显示任何保存在 leftCalloutAccessoryView 
 *      和 rightCalloutAccessoryView 属性中的自定义弹出视图。
 */
@property (nonatomic) BOOL canShowCallout;

/**
 *
 *  @brief   标准弹出气泡左侧显示的视图。
 *  @note   
 *      本属性默认的值为nil,左侧的弹出视图一般用来显示标注信息或者链接到你的应用提供的自定义信息。你的视图的高度应当小于32像
 *      素。
 *      \n如果你指定的视图还是UIControl类的派生，当控件被点击时你可以调用地图视图的委托来接收通知。如果不是派生自UIControl,
 *      你的视图负责处理任何它边界内的任何触摸事件。
 */
@property (retain, nonatomic) UIView *leftCalloutAccessoryView;

/**
 *
 *  @brief  标准弹出气泡右侧显示的视图。
 *  @see    canShowCallout
 *  @note   
 *      本属性默认的值为nil,左侧的弹出视图一般用来链接到更详细的标注信息。你的视图的高度应当小于32像素。一般指定到本属性的视
 *      图是类型为 UIButtonTypeDetailDisclosure 的在UIButton对象。
 *      \n如果你指定的视图还是UIControl类的派生，当控件被点击时你可以调用地图视图的委托接收通知。如果不是派生自UIControl,
 *      你的视图负责处理任何它边界内的任何触摸事件。
 */
@property (retain, nonatomic) UIView *rightCalloutAccessoryView;

/**
 *
 *  @brief  指示标注视图是否能被拖动的布尔值。
 *  @see    canShowCallout
 *  @note   
 *      设置本属性为YES使用户可以拖动标注。如果YES，关联的标注对象必须实现 setCoordinate: 方法。本属性默认的值为NO。
 *      \n设置本属性为YES使地图视图认为这个标注总是可以拖动的。换句话说，你不能在已经初始化的条件下尝试停止一个操作来限制某
 *      些拖动操作；这样做会导致出现未定义的行为。一旦拖动操作开始，就会一直执行直到完成。
 */
@property (nonatomic, getter=isDraggable) BOOL draggable;

/**
 *
 *  @brief  标注视图当前的拖动状态。
 *  @see    draggable
 *  @note   
 *      你需要重写 setDragState:animated: 方法来管理拖动状态。为了支持拖动操作，你必须重写这个属性的实现，同时按下面的
 *      说明来更新拖动状态：
 *      \n当拖动状态变为 MBAnnotationViewDragStateStarting 时，你应该把状态设为 MBAnnotationViewDragStateDragging 。
 *      如果通过执行动画来表示拖动的开始，你应该在状态改变前执行动画。改变状态到新的数值让地图知道你的动画执行完了。
 *      \n当拖动状态变为 MBAnnotationViewDragStateCanceling 或 MBAnnotationViewDragStateEnding时，
 *      你应该把状态设为 MBAnnotationViewDragStateNone 。如果通过执行动画来表示拖动的结束，你应该在状态改变前执行动画。
 *      \n把状态变为 MBAnnotationViewDragStateDragging 或者 MBAnnotationViewDragStateNone 是一种通知地图你
 *      想要执行动画的方式。例如，当拖动大头针标注的操作开始， MBPinAnnotationView 类将执行把大头针从地图提起的动画。同
 *      样的，当大头针被放下时，将执行放下的动画。即使你不想执行任何动画，你仍需要改变这个属性的值来反映正确的状态。
 *      \n你不要尝试通过把状态从 MBAnnotationViewDragStateStarting 变为 MBAnnotationViewDragStateNone 来取
 *      消一个新的拖动操作。如果你不希望标注视图被拖动，设置 draggable 属性为NO。
 */
@property (nonatomic) MBAnnotationViewDragState dragState;

/**
 *
 *  @brief  设置标注视图当前的拖动状态。
 *  @param newDragState 标注视图新的拖动状态。
 *  @param  animated    如果YES,到新拖动状态的变化是动画的，否则将没有动画。
 *  @note   
 *      应用需要重写本方法为自定义标注视图实现拖动支持。当系统检测到用户的拖动动作时，它调用本方法来更新拖动状态。为了应对这些
 *      改变，本方法自定义的实现要做到下面几点：
 *      \n当拖动状态变为 MBAnnotationViewDragStateStarting 时，你应该把状态设为 MBAnnotationViewDragStateDragging 。
 *      如果通过执行动画来表示拖动的开始，你应该在状态改变前执行动画。改变状态到新的值让地图知道你的动画执行完了。
 *      \n当拖动状态变为 MBAnnotationViewDragStateCanceling 或 MBAnnotationViewDragStateEnding 时，
 *      你应该把状态设为 MBAnnotationViewDragStateNone 。如果通过执行动画来表示拖动的结束，你应该在状态改变前执行动画。
 *      \n把状态变为 MBAnnotationViewDragStateDragging 或者 MBAnnotationViewDragStateNone 是一种通知地图你
 *      想要执行动画的方式。例如，当拖动大头针标注的操作开始， MBPinAnnotationView类将执行把大头针从地图提起的动画。同样
 *      的，当大头针被放下时，将执行放下的动画。即使你不想执行任何动画，你仍需要改变这个属性的值来反映正确的状态
 *      \n你不要尝试通过把状态从 MBAnnotationViewDragStateStarting 变为 MBAnnotationViewDragStateNone 来取
 *      消一个新的拖动操作。如果你不希望标注视图被拖动，设置 draggable 属性为NO。
 */
- (void)setDragState:(MBAnnotationViewDragState)newDragState animated:(BOOL)animated;

@end
