//
//  MBOverlayPathView.h
//  MBMapKit
//
//  Copyright 2011 Mapbar Inc. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "MBOverlayView.h"

/** @interface MBOverlayPathView
 *
 *  @brief
 *      MBOverlayPathView类使用CGPathRef数据类型用于绘制一般的覆盖层。你可以调用本方法来实现简单的基于路径的覆盖层视图，
 *      或者子类化它来定义额外的绘制行为。本类默认的绘制行为是应用对象当前的填充属性填充路径，应用当前的描边属性描绘路径。
 *      \n子类化时你需要重写 createPath 方法，使用该方法为覆盖层创建适当的路径。当你的子类获得新数据时你可以使当前路径无效
 *      并强制重新创建路径。
 */
@interface MBOverlayPathView : MBOverlayView 
{
@package
    UIColor *_fillColor;
    UIColor *_strokeColor;
    
    CGFloat _lineWidth;
    CGLineJoin _lineJoin;
    CGLineCap _lineCap;
    CGFloat _miterLimit;
    CGFloat _lineDashPhase;
    NSArray *_lineDashPattern;
    
    CGPathRef _path;
}

/** @property   fillColor
 *
 *  @brief  路径的填充色。 
 */
@property (retain) UIColor *fillColor;

/** @property   strokeColor
 *  
 *  @brief  路径的描边色。
 */
@property (retain) UIColor *strokeColor;

/** @property   lineWidth
 *
 *  @brief  路径的描边宽度。
 *  @note
 *      本属性的默认值为0。
 */
@property CGFloat lineWidth;

/** @property   lineJoin
 *
 *  @brief  应用到路径拐角的线条连结式样。
 *  @note
 *      默认线条连结式样为kCGLineJoinMiter。
 */
@property CGLineJoin lineJoin;

/** @property   lineCap
 *
 *  @brief  应用到开放路径的线帽式样。
 *  @note
 *      线帽式样将应用于任何子开放路径的起始点。本属性将不影响闭合子路径。默认线帽式样为kCGLineCapButt。
 */
@property CGLineCap lineCap;

/** @property   miterLimit
 *
 *  @brief  限制值用于避免连结线段拐角出现尖峰。
 *  @note
 *      斜角缝限制值用于当路径使用kCGLineJoinMiter连结式样时避免尖峰。如果斜角缝长度的比率（斜角缝连结处的对角线长）到线
 *      条宽度超过斜角缝限制时，接头被转化为斜面接头。默认斜角缝限制为10，也就是当接头夹角小于11度时会发生转化。
 */
@property CGFloat miterLimit;

/** @property   lineDashPhase
 *  
 *  @brief  指定虚线式样下线条的起点。
 */
@property CGFloat lineDashPhase; 

/** @property   lineDashPattern
 *
 *  @brief  一组数字用于指定路径的虚线式样。
 *  @note
 *      本数组包含了一个或多个对象用于指定线段的长度（以点为单位计算）和式样的空隙。数组中的数值轮流交替使用，开始是第一条线
 *      段的长度，接着是第一个空隙的长度，接着是第二条线段的长度，依此类推。
 *      \n本属性默认设置为nil,表示没有虚线式样。
 */
@property (copy) NSArray *lineDashPattern;

/**
 *
 *  @brief  创建覆盖层的路径。
 *  @note
 *      本方法的默认实现不执行任何操作。子类需要重写本方法用于创建用于绘制的CGPathRef类型。创建路径后，你的实现需要把它赋值
 *      给 path 属性。
 */
- (void)createPath;

/**
 *
 *  @brief  用于绘制覆盖层的当前路径。
 *  @see    createPath 
 *  @see    invalidatePath
 *  @note
 *      调用本属性时如果路径不存在将促使路径被创建（调用 createPath 方法）。你可以直接给本属性精确赋一个路径对象。
 *      \n当给本属性赋一个新的路径对象时，接收者将保留你指定的对象。 
 */
@property CGPathRef path;

/** 
 *
 *  @brief  释放与接收者关联的路径。
 *  @note   
 *      当路径信息变化并要求你重新创建路径时你可以调用本方法。本方法设置 path 属性为nil, 以致缓存的路径被释放。
 */
- (void)invalidatePath;

/**
 *
 *  @brief  应用接收者当前填充相关的绘制属性到指定的图像上下文。
 *  @param  context 用于绘制视图内容的图形上下文。
 *  @param  zoomScale   绘制时当前缩放等级。
 *  @see    fillPath:inContext:
 *  @note
 *      填充路径的时候这是一个便捷的方法应用所有绘制属性。本方法应用当前填充色到指定的图形上下文。
 */
- (void)applyStrokePropertiesToContext:(CGContextRef)context
                           atZoomScale:(MBZoomScale)zoomScale;

/** 
 *
 *  @brief  应用接收者当前描边相关的绘制属性到指定的图像上下文。
 *  @param  context 用于绘制视图内容的图形上下文。
 *  @param  zoomScale   绘制时当前缩放等级。
 *  @see    strokePath:inContext:
 *  @note
 *      描边路径的时候这是一个便捷的方法应用所有绘制属性。本方法应用描边色、线宽、线段连结、线帽、斜角缝、虚线起点、虚线属性
 *      到到指定的图形上下文。本方法应用缩放因子到线宽及虚线式样中使线条适当缩放。
 *      \n本方法在应用新属性前不保存当前的图形状态。你必须自己保存，当你完成绘制后再恢复它。
 */
- (void)applyFillPropertiesToContext:(CGContextRef)context
                         atZoomScale:(MBZoomScale)zoomScale;

/**
 *
 *  @brief  沿着指定的路径绘制一条线。
 *  @param  path    将要绘制的路径.
 *  @param  context 绘制路径的图形上下文。
 *  @see    applyStrokePropertiesToContext:atZoomScale:
 *  @note
 *      调用本方法前你必须设置当前的描边色。通常你先调用 applyStrokePropertiesToContext:atZoomScale: 方法再执行本
 *      方法用于绘制。如果 strokeColor 属性为nil，本方法不执行任何操作。
 */
- (void)strokePath:(CGPathRef)path inContext:(CGContextRef)context;

/**
 *
 *  @brief  填充指定的路径围成的区域。
 *  @param  path    将要填充的路径。
 *  @param  context 绘制路径的图形上下文。
 *  @see    applyFillPropertiesToContext:atZoomScale:
 *  @note
 *      调用本方法前你必须设置当前的填充色。通常你先调用 applyFillPropertiesToContext:atZoomScale: 方法再执行本方
 *      法用于绘制。如果 fillColor 属性为nil，本方法不执行任何操作。
 */
- (void)fillPath:(CGPathRef)path inContext:(CGContextRef)context;

@end
